"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var MongodbModule_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongodbModule = void 0;
const common_1 = require("@nestjs/common");
const mongodb_1 = require("mongodb");
const node_crypto_1 = require("node:crypto");
const data_source_1 = require("../lib/data-source");
const utils_1 = require("../lib/utils");
const constants_1 = require("../lib/constants");
const mongodb_service_1 = require("./mongodb.service");
let MongodbModule = MongodbModule_1 = class MongodbModule {
    static forRoot({ global, connectionName, options = {} }) {
        const defaultOptions = (0, utils_1.getDefaultDataSourceOptions)(options);
        const uri = (0, utils_1.getConnectionString)(defaultOptions);
        const clientToken = (0, utils_1.getClientToken)(connectionName);
        const dataSourceToken = (0, utils_1.getDataSourceToken)(connectionName);
        return {
            global: global ?? true,
            module: MongodbModule_1,
            exports: [clientToken, dataSourceToken],
            providers: [
                {
                    provide: clientToken,
                    useFactory: () => new mongodb_1.MongoClient(uri, defaultOptions.clientOptions)
                },
                {
                    provide: dataSourceToken,
                    inject: [clientToken],
                    useFactory: (client) => new data_source_1.DataSource(client, defaultOptions)
                },
                {
                    provide: constants_1.INTERNAL_DATA_SOURCE,
                    inject: [dataSourceToken],
                    useFactory: (dataSource) => dataSource
                },
                {
                    provide: (0, node_crypto_1.randomUUID)(),
                    useClass: mongodb_service_1.MongodbService
                }
            ]
        };
    }
    static forRootAsync({ global, connectionName, inject, useFactory }) {
        const clientToken = (0, utils_1.getClientToken)(connectionName);
        const dataSourceToken = (0, utils_1.getDataSourceToken)(connectionName);
        const dataSourceOptionsToken = (0, node_crypto_1.randomUUID)();
        return {
            global: global ?? true,
            module: MongodbModule_1,
            exports: [clientToken, dataSourceToken],
            providers: [
                {
                    provide: dataSourceOptionsToken,
                    inject,
                    useFactory
                },
                {
                    provide: clientToken,
                    inject: [dataSourceOptionsToken],
                    useFactory: (options) => {
                        const defaultOptions = (0, utils_1.getDefaultDataSourceOptions)(options);
                        const uri = (0, utils_1.getConnectionString)(defaultOptions);
                        return new mongodb_1.MongoClient(uri, defaultOptions.clientOptions);
                    }
                },
                {
                    provide: dataSourceToken,
                    inject: [clientToken, dataSourceOptionsToken],
                    useFactory: (client, options) => {
                        const defaultOptions = (0, utils_1.getDefaultDataSourceOptions)(options);
                        return new data_source_1.DataSource(client, defaultOptions);
                    }
                },
                {
                    provide: constants_1.INTERNAL_DATA_SOURCE,
                    inject: [dataSourceToken],
                    useFactory: (dataSource) => dataSource
                },
                {
                    provide: (0, node_crypto_1.randomUUID)(),
                    useClass: mongodb_service_1.MongodbService
                }
            ]
        };
    }
    static forFeature(entities, connectionName) {
        const dataSourceToken = (0, utils_1.getDataSourceToken)(connectionName);
        const providers = entities.map(entity => ({
            provide: (0, utils_1.getCollectionToken)(entity, connectionName),
            inject: [dataSourceToken],
            useFactory: (dataSource) => dataSource.getCollection(entity)
        }));
        return {
            module: MongodbModule_1,
            providers,
            exports: providers.map(({ provide }) => provide)
        };
    }
    static forTest() {
        return {
            module: MongodbModule_1,
            providers: [],
            exports: []
        };
    }
};
exports.MongodbModule = MongodbModule;
exports.MongodbModule = MongodbModule = MongodbModule_1 = __decorate([
    (0, common_1.Module)({})
], MongodbModule);
//# sourceMappingURL=mongodb.module.js.map